@@ -1,4 +1,4 @@
-"""AMQP Connections"""
+"""AMQP Connections."""
 # Copyright (C) 2007-2008 Barry Pederson <bp@barryp.org>
 #
 # This library is free software; you can redistribute it and/or
@@ -34,7 +34,7 @@ from .exceptions import (
     ConnectionForced, ConnectionError, error_for_code,
     RecoverableConnectionError, RecoverableChannelError,
 )
-from .five import array, range, values, monotonic
+from .five import array, items, monotonic, range, values
 from .method_framing import frame_handler, frame_writer
 from .serialization import _write_table
 from .transport import Transport
@@ -60,20 +60,26 @@ Start from server, version: %d.%d, properties: %s, mechanisms: %s, locales: %s
 
 __all__ = ['Connection']
 
-#
-# Client property info that gets sent to the server on connection startup
-#
+AMQP_LOGGER = logging.getLogger('amqp')
+
+#: Default map for :attr:`Connection.library_properties`
 LIBRARY_PROPERTIES = {
     'product': 'py-amqp',
     'product_version': __version__,
-    'capabilities': {},
 }
 
-AMQP_LOGGER = logging.getLogger('amqp')
+#: Default map for :attr:`Connection.negotiate_capabilities`
+NEGOTIATE_CAPABILITIES = {
+    'consumer_cancel_notify': True,
+    'connection.blocked': True,
+    'authentication_failure_close': True,
+}
 
 
 class Connection(AbstractChannel):
-    """The connection class provides methods for a client to establish a
+    """AMQP Connection.
+
+    The connection class provides methods for a client to establish a
     network connection to a server, and for both peers to operate the
     connection thereafter.
 
@@ -89,10 +95,42 @@ class Connection(AbstractChannel):
         use-connection      = *channel
         close-connection    = C:CLOSE S:CLOSE-OK
                             / S:CLOSE C:CLOSE-OK
+    Create a connection to the specified host, which should be
+    a 'host[:port]', such as 'localhost', or '1.2.3.4:5672'
+    (defaults to 'localhost', if a port is not specified then
+    5672 is used)
 
+    If login_response is not specified, one is built up for you from
+    userid and password if they are present.
+
+    The 'ssl' parameter may be simply True/False, or for Python >= 2.6
+    a dictionary of options to pass to ssl.wrap_socket() such as
+    requiring certain certificates.
+
+    The "socket_settings" parameter is a dictionary defining tcp
+    settings which will be applied as socket options.
     """
+
     Channel = Channel
 
+    #: Mapping of protocol extensions to enable.
+    #: The server will report these in server_properties[capabilities],
+    #: and if a key in this map is present the client will tell the
+    #: server to either enable or disable the capability depending
+    #: on the value set in this map.
+    #: For example with:
+    #:     negotiate_capabilities = {
+    #:         'consumer_cancel_notify': True,
+    #:     }
+    #: The client will enable this capability if the server reports
+    #: support for it, but if the value is False the client will
+    #: disable the capability.
+    negotiate_capabilities = NEGOTIATE_CAPABILITIES
+
+    #: These are sent to the server to announce what features
+    #: we support, type of client etc.
+    library_properties = LIBRARY_PROPERTIES
+
     #: Final heartbeat interval value (in float seconds) after negotiation
     heartbeat = None
 
@@ -158,22 +196,6 @@ class Connection(AbstractChannel):
                  on_tune_ok=None, read_timeout=None, write_timeout=None,
                  socket_settings=None, frame_handler=frame_handler,
                  frame_writer=frame_writer, **kwargs):
-        """Create a connection to the specified host, which should be
-        a 'host[:port]', such as 'localhost', or '1.2.3.4:5672'
-        (defaults to 'localhost', if a port is not specified then
-        5672 is used)
-
-        If login_response is not specified, one is built up for you from
-        userid and password if they are present.
-
-        The 'ssl' parameter may be simply True/False, or for Python >= 2.6
-        a dictionary of options to pass to ssl.wrap_socket() such as
-        requiring certain certificates.
-
-        The "socket_settings" parameter is a dictionary defining tcp
-        settings which will be applied as socket options.
-
-        """
         self._connection_id = uuid.uuid4().hex
         channel_max = channel_max or 65535
         frame_max = frame_max or 131072
@@ -187,7 +209,7 @@ class Connection(AbstractChannel):
             login_response = login_response.getvalue()[4:]
 
         self.client_properties = dict(
-            LIBRARY_PROPERTIES, **client_properties or {}
+            self.library_properties, **client_properties or {}
         )
         self.login_method = login_method
         self.login_response = login_response
@@ -328,14 +350,18 @@ class Connection(AbstractChannel):
             self.server_properties, self.mechanisms, self.locales,
         )
 
+        # Negotiate protocol extensions (capabilities)
         scap = server_properties.get('capabilities') or {}
-
-        if scap.get('consumer_cancel_notify'):
-            cap = client_properties.setdefault('capabilities', {})
-            cap['consumer_cancel_notify'] = True
-        if scap.get('connection.blocked'):
-            cap = client_properties.setdefault('capabilities', {})
-            cap['connection.blocked'] = True
+        cap = client_properties.setdefault('capabilities', {})
+        cap.update({
+            wanted_cap: enable_cap
+            for wanted_cap, enable_cap in items(self.negotiate_capabilities)
+            if scap.get(wanted_cap)
+        })
+        if not cap:
+            # no capabilities, server may not react well to having
+            # this key present in client_properties, so we remove it.
+            client_properties.pop('capabilities', None)
 
         self.send_method(
             spec.Connection.StartOk, argsig,
@@ -417,8 +443,11 @@ class Connection(AbstractChannel):
             raise ConnectionError('Channel %r already open' % (channel_id,))
 
     def channel(self, channel_id=None, callback=None):
-        """Fetch a Channel object identified by the numeric channel_id, or
-        create that object if it doesn't already exist."""
+        """Create new channel.
+
+        Fetch a Channel object identified by the numeric channel_id, or
+        create that object if it doesn't already exist.
+        """
         if self.channels is not None:
             try:
                 return self.channels[channel_id]
@@ -446,8 +475,7 @@ class Connection(AbstractChannel):
 
     def close(self, reply_code=0, reply_text='', method_sig=(0, 0),
               argsig='BsBB'):
-
-        """Request a connection close
+        """Request a connection close.
 
         This method indicates that the sender wants to close the
         connection. This may be due to internal conditions (e.g. a
@@ -498,7 +526,6 @@ class Connection(AbstractChannel):
 
                 When the close is provoked by a method exception, this
                 is the ID of the method.
-
         """
         if self._transport is None:
             # already closed
@@ -511,7 +538,7 @@ class Connection(AbstractChannel):
         )
 
     def _on_close(self, reply_code, reply_text, class_id, method_id):
-        """Request a connection close
+        """Request a connection close.
 
         This method indicates that the sender wants to close the
         connection. This may be due to internal conditions (e.g. a
@@ -562,29 +589,26 @@ class Connection(AbstractChannel):
 
                 When the close is provoked by a method exception, this
                 is the ID of the method.
-
         """
         self._x_close_ok()
         raise error_for_code(reply_code, reply_text,
                              (class_id, method_id), ConnectionError)
 
     def _x_close_ok(self):
-        """Confirm a connection close
+        """Confirm a connection close.
 
         This method confirms a Connection.Close method and tells the
         recipient that it is safe to release resources for the
         connection and close the socket.
 
         RULE:
-
             A peer that detects a socket closure without having
             received a Close-Ok handshake method SHOULD log the error.
-
         """
         self.send_method(spec.Connection.CloseOk, callback=self._on_close_ok)
 
     def _on_close_ok(self):
-        """Confirm a connection close
+        """Confirm a connection close.
 
         This method confirms a Connection.Close method and tells the
         recipient that it is safe to release resources for the
@@ -594,12 +618,15 @@ class Connection(AbstractChannel):
 
             A peer that detects a socket closure without having
             received a Close-Ok handshake method SHOULD log the error.
-
         """
         self.collect()
 
     def _on_blocked(self):
-        """RabbitMQ Extension."""
+        """Callback called when connection blocked.
+
+        Notes:
+            This is an RabbitMQ Extension.
+        """
         reason = 'connection blocked, see broker logs'
         if self.on_blocked:
             return self.on_blocked(reason)
@@ -612,11 +639,18 @@ class Connection(AbstractChannel):
         self.frame_writer(8, 0, None, None, None)
 
     def heartbeat_tick(self, rate=2):
-        """Send heartbeat packets, if necessary, and fail if none have been
-        received recently.  This should be called frequently, on the order of
-        once per second.
+        """Send heartbeat packets if necessary.
 
-        :keyword rate: Ignored
+        Raises:
+            ~amqp.exceptions.ConnectionForvced: if none have been
+                received recently.
+
+        Note:
+            This should be called frequently, on the order of
+            once per second.
+
+        Keyword Arguments:
+            rate (int): Previously used, but ignored now.
         """
         AMQP_LOGGER.debug('heartbeat_tick : for connection %s',
                           self._connection_id)
