@@ -1,3 +1,4 @@
+"""Transport implementation."""
 # Copyright (C) 2009 Barry Pederson <bp@barryp.org>
 #
 # This library is free software; you can redistribute it and/or
@@ -17,28 +18,24 @@ from __future__ import absolute_import, unicode_literals
 
 import errno
 import re
-import struct
 import socket
 import ssl
 
 from contextlib import contextmanager
-from struct import unpack
 
 from .exceptions import UnexpectedFrame
 from .five import items
+from .platform import (
+    SOL_TCP, TCP_USER_TIMEOUT, HAS_TCP_USER_TIMEOUT,
+    pack, unpack,
+)
 from .utils import get_errno, set_cloexec
 
-# Jython does not have this attribute
-try:
-    from socket import SOL_TCP
-except ImportError:  # pragma: no cover
-    from socket import IPPROTO_TCP as SOL_TCP  # noqa
-
 try:
     from ssl import SSLError
 except ImportError:  # pragma: no cover
     class SSLError(Exception):  # noqa
-        pass
+        """Dummy SSL exception."""
 
 _UNAVAIL = {errno.EAGAIN, errno.EINTR, errno.ENOENT, errno.EWOULDBLOCK}
 
@@ -61,11 +58,33 @@ KNOWN_TCP_OPTS = (
     'TCP_MAXSEG', 'TCP_NODELAY', 'TCP_QUICKACK',
     'TCP_SYNCNT', 'TCP_WINDOW_CLAMP',
 )
-TCP_OPTS = [getattr(socket, opt) for opt in KNOWN_TCP_OPTS
-            if hasattr(socket, opt)]
+TCP_OPTS = {
+    getattr(socket, opt) for opt in KNOWN_TCP_OPTS if hasattr(socket, opt)
+}
+DEFAULT_SOCKET_SETTINGS = {
+    socket.TCP_NODELAY: 1,
+}
+
+if HAS_TCP_USER_TIMEOUT:
+    KNOWN_TCP_OPTS += ('TCP_USER_TIMEOUT',)
+    TCP_OPTS.add(TCP_USER_TIMEOUT)
+    DEFAULT_SOCKET_SETTINGS[TCP_USER_TIMEOUT] = 1000
+
+
+try:
+    from socket import TCP_KEEPIDLE, TCP_KEEPINTVL, TCP_KEEPCNT # noqa
+except ImportError:
+    pass
+else:
+    DEFAULT_SOCKET_SETTINGS.update({
+        TCP_KEEPIDLE: 60,
+        TCP_KEEPINTVL: 10,
+        TCP_KEEPCNT: 9,
+    })
 
 
 def to_host_port(host, default=AMQP_PORT):
+    """Convert hostname:port string to host, port tuple."""
     port = default
     m = IPV6_LITERAL.match(host)
     if m:
@@ -80,7 +99,8 @@ def to_host_port(host, default=AMQP_PORT):
 
 
 class _AbstractTransport(object):
-    """Common superclass for TCP and SSL transports"""
+    """Common superclass for TCP and SSL transports."""
+
     connected = False
 
     def __init__(self, host, connect_timeout=None,
@@ -159,7 +179,7 @@ class _AbstractTransport(object):
                 if interval is not None:
                     self.sock.setsockopt(
                         socket.SOL_SOCKET, timeout,
-                        struct.pack('ll', interval, 0),
+                        pack('ll', interval, 0),
                     )
             self._setup_transport()
 
@@ -175,24 +195,20 @@ class _AbstractTransport(object):
         }
 
     def _set_socket_options(self, socket_settings):
-        if not socket_settings:
-            self.sock.setsockopt(SOL_TCP, socket.TCP_NODELAY, 1)
-            return
-
         tcp_opts = self._get_tcp_socket_defaults(self.sock)
-        tcp_opts.setdefault(socket.TCP_NODELAY, 1)
-        tcp_opts.update(socket_settings)
-
+        final_socket_settings = dict(DEFAULT_SOCKET_SETTINGS)
+        if socket_settings:
+            final_socket_settings.update(socket_settings)
+        tcp_opts.update(final_socket_settings)
         for opt, val in items(tcp_opts):
             self.sock.setsockopt(SOL_TCP, opt, val)
 
     def _read(self, n, initial=False):
-        """Read exactly n bytes from the peer"""
+        """Read exactly n bytes from the peer."""
         raise NotImplementedError('Must be overriden in subclass')
 
     def _setup_transport(self):
-        """Do any additional initialization of the class (used
-        by the subclasses)."""
+        """Do any additional initialization of the class."""
         pass
 
     def _shutdown_transport(self):
@@ -220,7 +236,7 @@ class _AbstractTransport(object):
         try:
             frame_header = read(7, True)
             read_frame_buffer += frame_header
-            frame_type, channel, size = unpack(u'>BHI', frame_header)
+            frame_type, channel, size = unpack('>BHI', frame_header)
             # >I is an unsigned int, but the argument to sock.recv is signed,
             # so we know the size can be at most 2 * SIGNED_INT_MAX
             if size > SIGNED_INT_MAX:
@@ -260,7 +276,7 @@ class _AbstractTransport(object):
 
 
 class SSLTransport(_AbstractTransport):
-    """Transport that works over SSL"""
+    """Transport that works over SSL."""
 
     def __init__(self, host, connect_timeout=None, ssl=None, **kwargs):
         self.sslopts = ssl if isinstance(ssl, dict) else {}
@@ -285,7 +301,7 @@ class SSLTransport(_AbstractTransport):
         return ctx.wrap_socket(sock, **sslopts)
 
     def _shutdown_transport(self):
-        """Unwrap a Python 2.6 SSL socket, so we can call shutdown()"""
+        """Unwrap a Python 2.6 SSL socket, so we can call shutdown()."""
         if self.sock is not None:
             try:
                 unwrap = self.sock.unwrap
@@ -345,14 +361,14 @@ class TCPTransport(_AbstractTransport):
     """Transport that deals directly with TCP socket."""
 
     def _setup_transport(self):
-        """Setup to _write() directly to the socket, and
-        do our own buffered reads."""
+        # Setup to _write() directly to the socket, and
+        # do our own buffered reads.
         self._write = self.sock.sendall
         self._read_buffer = EMPTY_BUFFER
         self._quick_recv = self.sock.recv
 
     def _read(self, n, initial=False, _errnos=(errno.EAGAIN, errno.EINTR)):
-        """Read exactly n bytes from the socket"""
+        """Read exactly n bytes from the socket."""
         recv = self._quick_recv
         rbuf = self._read_buffer
         try:
@@ -377,7 +393,10 @@ class TCPTransport(_AbstractTransport):
 
 
 def Transport(host, connect_timeout=None, ssl=False, **kwargs):
-    """Given a few parameters from the Connection constructor,
-    select and create a subclass of _AbstractTransport."""
+    """Create transport.
+
+    Given a few parameters from the Connection constructor,
+    select and create a subclass of _AbstractTransport.
+    """
     transport = SSLTransport if ssl else TCPTransport
     return transport(host, connect_timeout=connect_timeout, ssl=ssl, **kwargs)
